@@ -11,13 +11,14 @@ from xarray import (
     Dataset,
     SerializationWarning,
     Variable,
-    cftime_range,
     coding,
     conventions,
+    date_range,
     open_dataset,
 )
 from xarray.backends.common import WritableCFDataStore
 from xarray.backends.memory import InMemoryDataStore
+from xarray.coders import CFDatetimeCoder, CFTimedeltaCoder
 from xarray.conventions import decode_cf
 from xarray.testing import assert_identical
 from xarray.tests import (
@@ -52,10 +53,9 @@ def test_decode_cf_with_conflicting_fill_missing_value() -> None:
     var = Variable(
         ["t"], np.arange(3), {"units": "foobar", "missing_value": 0, "_FillValue": 1}
     )
-    with warnings.catch_warnings(record=True) as w:
+    with pytest.warns(SerializationWarning, match="has multiple fill"):
         actual = conventions.decode_cf_variable("t", var)
         assert_identical(actual, expected)
-        assert "has multiple fill" in str(w[0].message)
 
     expected = Variable(["t"], np.arange(10), {"units": "foobar"})
 
@@ -64,7 +64,13 @@ def test_decode_cf_with_conflicting_fill_missing_value() -> None:
         np.arange(10),
         {"units": "foobar", "missing_value": np.nan, "_FillValue": np.nan},
     )
-    actual = conventions.decode_cf_variable("t", var)
+
+    # the following code issues two warnings, so we need to check for both
+    with pytest.warns(SerializationWarning) as winfo:
+        actual = conventions.decode_cf_variable("t", var)
+    for aw in winfo:
+        assert "non-conforming" in str(aw.message)
+
     assert_identical(actual, expected)
 
     var = Variable(
@@ -76,10 +82,37 @@ def test_decode_cf_with_conflicting_fill_missing_value() -> None:
             "_FillValue": np.float32(np.nan),
         },
     )
-    actual = conventions.decode_cf_variable("t", var)
+
+    # the following code issues two warnings, so we need to check for both
+    with pytest.warns(SerializationWarning) as winfo:
+        actual = conventions.decode_cf_variable("t", var)
+    for aw in winfo:
+        assert "non-conforming" in str(aw.message)
     assert_identical(actual, expected)
 
 
+def test_decode_cf_variable_with_mismatched_coordinates() -> None:
+    # tests for decoding mismatched coordinates attributes
+    # see GH #1809
+    zeros1 = np.zeros((1, 5, 3))
+    orig = Dataset(
+        {
+            "XLONG": (["x", "y"], zeros1.squeeze(0), {}),
+            "XLAT": (["x", "y"], zeros1.squeeze(0), {}),
+            "foo": (["time", "x", "y"], zeros1, {"coordinates": "XTIME XLONG XLAT"}),
+            "time": ("time", [0.0], {"units": "hours since 2017-01-01"}),
+        }
+    )
+    decoded = conventions.decode_cf(orig, decode_coords=True)
+    assert decoded["foo"].encoding["coordinates"] == "XTIME XLONG XLAT"
+    assert list(decoded.coords.keys()) == ["XLONG", "XLAT", "time"]
+
+    decoded = conventions.decode_cf(orig, decode_coords=False)
+    assert "coordinates" not in decoded["foo"].encoding
+    assert decoded["foo"].attrs.get("coordinates") == "XTIME XLONG XLAT"
+    assert list(decoded.coords.keys()) == ["time"]
+
+
 @requires_cftime
 class TestEncodeCFVariable:
     def test_incompatible_attributes(self) -> None:
@@ -87,7 +120,7 @@ class TestEncodeCFVariable:
             Variable(
                 ["t"], pd.date_range("2000-01-01", periods=3), {"units": "foobar"}
             ),
-            Variable(["t"], pd.to_timedelta(["1 day"]), {"units": "foobar"}),
+            Variable(["t"], pd.to_timedelta(["1 day"]), {"units": "foobar"}),  # type: ignore[arg-type, unused-ignore]
             Variable(["t"], [0, 1, 2], {"add_offset": 0}, {"add_offset": 2}),
             Variable(["t"], [0, 1, 2], {"_FillValue": 0}, {"_FillValue": 2}),
         ]
@@ -129,9 +162,9 @@ class TestEncodeCFVariable:
         foo1_coords = enc["foo1"].attrs.get("coordinates", "")
         foo2_coords = enc["foo2"].attrs.get("coordinates", "")
         foo3_coords = enc["foo3"].attrs.get("coordinates", "")
-        assert set(foo1_coords.split()) == {"lat1", "lon1"}
-        assert set(foo2_coords.split()) == {"lat2", "lon2"}
-        assert set(foo3_coords.split()) == {"lat3", "lon3"}
+        assert foo1_coords == "lon1 lat1"
+        assert foo2_coords == "lon2 lat2"
+        assert foo3_coords == "lon3 lat3"
         # Should not have any global coordinates.
         assert "coordinates" not in attrs
 
@@ -150,11 +183,12 @@ class TestEncodeCFVariable:
         enc, attrs = conventions.encode_dataset_coordinates(orig)
         # Make sure we have the right coordinates for each variable.
         values_coords = enc["values"].attrs.get("coordinates", "")
-        assert set(values_coords.split()) == {"time", "lat", "lon"}
+        assert values_coords == "time lon lat"
         # Should not have any global coordinates.
         assert "coordinates" not in attrs
 
     def test_do_not_overwrite_user_coordinates(self) -> None:
+        # don't overwrite user-defined "coordinates" encoding
         orig = Dataset(
             coords={"x": [0, 1, 2], "y": ("x", [5, 6, 7]), "z": ("x", [8, 9, 10])},
             data_vars={"a": ("x", [1, 2, 3]), "b": ("x", [3, 5, 6])},
@@ -168,7 +202,18 @@ class TestEncodeCFVariable:
         with pytest.raises(ValueError, match=r"'coordinates' found in both attrs"):
             conventions.encode_dataset_coordinates(orig)
 
-    @pytest.mark.filterwarnings("ignore:Converting non-nanosecond")
+    def test_deterministic_coords_encoding(self) -> None:
+        # the coordinates attribute is sorted when set by xarray.conventions ...
+        # ... on a variable's coordinates attribute
+        ds = Dataset({"foo": 0}, coords={"baz": 0, "bar": 0})
+        vars, attrs = conventions.encode_dataset_coordinates(ds)
+        assert vars["foo"].attrs["coordinates"] == "bar baz"
+        assert attrs.get("coordinates") is None
+        # ... on the global coordinates attribute
+        ds = ds.drop_vars("foo")
+        vars, attrs = conventions.encode_dataset_coordinates(ds)
+        assert attrs["coordinates"] == "bar baz"
+
     def test_emit_coordinates_attribute_in_attrs(self) -> None:
         orig = Dataset(
             {"a": 1, "b": 1},
@@ -186,7 +231,6 @@ class TestEncodeCFVariable:
         assert enc["b"].attrs.get("coordinates") == "t"
         assert "coordinates" not in enc["b"].encoding
 
-    @pytest.mark.filterwarnings("ignore:Converting non-nanosecond")
     def test_emit_coordinates_attribute_in_encoding(self) -> None:
         orig = Dataset(
             {"a": 1, "b": 1},
@@ -204,13 +248,6 @@ class TestEncodeCFVariable:
         assert enc["b"].attrs.get("coordinates") == "t"
         assert "coordinates" not in enc["b"].encoding
 
-    @requires_dask
-    def test_string_object_warning(self) -> None:
-        original = Variable(("x",), np.array(["foo", "bar"], dtype=object)).chunk()
-        with pytest.warns(SerializationWarning, match="dask array with dtype=object"):
-            encoded = conventions.encode_cf_variable(original)
-        assert_identical(original, encoded)
-
 
 @requires_cftime
 class TestDecodeCF:
@@ -233,9 +270,12 @@ class TestDecodeCF:
         assert_identical(expected, actual)
 
     def test_invalid_coordinates(self) -> None:
-        # regression test for GH308
+        # regression test for GH308, GH1809
         original = Dataset({"foo": ("t", [1, 2], {"coordinates": "invalid"})})
+        decoded = Dataset({"foo": ("t", [1, 2], {}, {"coordinates": "invalid"})})
         actual = conventions.decode_cf(original)
+        assert_identical(decoded, actual)
+        actual = conventions.decode_cf(original, decode_coords=False)
         assert_identical(original, actual)
 
     def test_decode_coordinates(self) -> None:
@@ -246,6 +286,96 @@ class TestDecodeCF:
         actual = conventions.decode_cf(original)
         assert actual.foo.encoding["coordinates"] == "x"
 
+    def test_decode_coordinates_with_key_values(self) -> None:
+        # regression test for GH9761
+        original = Dataset(
+            {
+                "temp": (
+                    ("y", "x"),
+                    np.random.rand(2, 2),
+                    {
+                        "long_name": "temperature",
+                        "units": "K",
+                        "coordinates": "lat lon",
+                        "grid_mapping": "crs",
+                    },
+                ),
+                "x": (
+                    ("x"),
+                    np.arange(2),
+                    {"standard_name": "projection_x_coordinate", "units": "m"},
+                ),
+                "y": (
+                    ("y"),
+                    np.arange(2),
+                    {"standard_name": "projection_y_coordinate", "units": "m"},
+                ),
+                "lat": (
+                    ("y", "x"),
+                    np.random.rand(2, 2),
+                    {"standard_name": "latitude", "units": "degrees_north"},
+                ),
+                "lon": (
+                    ("y", "x"),
+                    np.random.rand(2, 2),
+                    {"standard_name": "longitude", "units": "degrees_east"},
+                ),
+                "crs": (
+                    (),
+                    None,
+                    {
+                        "grid_mapping_name": "transverse_mercator",
+                        "longitude_of_central_meridian": -2.0,
+                    },
+                ),
+                "crs2": (
+                    (),
+                    None,
+                    {
+                        "grid_mapping_name": "longitude_latitude",
+                        "longitude_of_central_meridian": -2.0,
+                    },
+                ),
+            },
+        )
+
+        original.temp.attrs["grid_mapping"] = "crs: x y"
+        vars, attrs, coords = conventions.decode_cf_variables(
+            original.variables, {}, decode_coords="all"
+        )
+        assert coords == {"lat", "lon", "crs"}
+
+        original.temp.attrs["grid_mapping"] = "crs: x y crs2: lat lon"
+        vars, attrs, coords = conventions.decode_cf_variables(
+            original.variables, {}, decode_coords="all"
+        )
+        assert coords == {"lat", "lon", "crs", "crs2"}
+
+        # stray colon
+        original.temp.attrs["grid_mapping"] = "crs: x y crs2 : lat lon"
+        vars, attrs, coords = conventions.decode_cf_variables(
+            original.variables, {}, decode_coords="all"
+        )
+        assert coords == {"lat", "lon", "crs", "crs2"}
+
+        original.temp.attrs["grid_mapping"] = "crs x y crs2: lat lon"
+        with pytest.raises(ValueError, match="misses ':'"):
+            conventions.decode_cf_variables(original.variables, {}, decode_coords="all")
+
+        del original.temp.attrs["grid_mapping"]
+        original.temp.attrs["formula_terms"] = "A: lat D: lon E: crs2"
+        vars, attrs, coords = conventions.decode_cf_variables(
+            original.variables, {}, decode_coords="all"
+        )
+        assert coords == {"lat", "lon", "crs2"}
+
+        original.temp.attrs["formula_terms"] = "A: lat lon D: crs E: crs2"
+        with pytest.warns(UserWarning, match="has malformed content"):
+            vars, attrs, coords = conventions.decode_cf_variables(
+                original.variables, {}, decode_coords="all"
+            )
+            assert coords == {"lat", "lon", "crs", "crs2"}
+
     def test_0d_int32_encoding(self) -> None:
         original = Variable((), np.int32(0), encoding={"dtype": "int64"})
         expected = Variable((), np.int64(0))
@@ -255,10 +385,9 @@ class TestDecodeCF:
     def test_decode_cf_with_multiple_missing_values(self) -> None:
         original = Variable(["t"], [0, 1, 2], {"missing_value": np.array([0, 1])})
         expected = Variable(["t"], [np.nan, np.nan, 2], {})
-        with warnings.catch_warnings(record=True) as w:
+        with pytest.warns(SerializationWarning, match="has multiple fill"):
             actual = conventions.decode_cf_variable("t", original)
             assert_identical(expected, actual)
-            assert "has multiple fill" in str(w[0].message)
 
     def test_decode_cf_with_drop_variables(self) -> None:
         original = Dataset(
@@ -295,8 +424,20 @@ class TestDecodeCF:
         with pytest.raises(ValueError, match=r"unable to decode time"):
             decode_cf(ds)
 
+    @pytest.mark.parametrize("decode_times", [True, False])
+    def test_invalid_timedelta_units_do_not_decode(self, decode_times) -> None:
+        # regression test for #8269
+        ds = Dataset(
+            {"time": ("time", [0, 1, 20], {"units": "days invalid", "_FillValue": 20})}
+        )
+        expected = Dataset(
+            {"time": ("time", [0.0, 1.0, np.nan], {"units": "days invalid"})}
+        )
+        assert_identical(expected, decode_cf(ds, decode_times=decode_times))
+
     @requires_cftime
-    def test_dataset_repr_with_netcdf4_datetimes(self) -> None:
+    @pytest.mark.parametrize("time_unit", ["s", "ms", "us", "ns"])
+    def test_dataset_repr_with_netcdf4_datetimes(self, time_unit) -> None:
         # regression test for #347
         attrs = {"units": "days since 0001-01-01", "calendar": "noleap"}
         with warnings.catch_warnings():
@@ -305,8 +446,11 @@ class TestDecodeCF:
             assert "(time) object" in repr(ds)
 
         attrs = {"units": "days since 1900-01-01"}
-        ds = decode_cf(Dataset({"time": ("time", [0, 1], attrs)}))
-        assert "(time) datetime64[ns]" in repr(ds)
+        ds = decode_cf(
+            Dataset({"time": ("time", [0, 1], attrs)}),
+            decode_times=CFDatetimeCoder(time_unit=time_unit),
+        )
+        assert f"(time) datetime64[{time_unit}]" in repr(ds)
 
     @requires_cftime
     def test_decode_cf_datetime_transition_to_invalid(self) -> None:
@@ -338,7 +482,6 @@ class TestDecodeCF:
             }
         ).chunk()
         decoded = conventions.decode_cf(original)
-        print(decoded)
         assert all(
             isinstance(var.data, da.Array)
             for name, var in decoded.variables.items()
@@ -366,7 +509,8 @@ class TestDecodeCF:
             conventions.decode_cf(original).chunk(),
         )
 
-    def test_decode_cf_time_kwargs(self) -> None:
+    @pytest.mark.parametrize("time_unit", ["s", "ms", "us", "ns"])
+    def test_decode_cf_time_kwargs(self, time_unit) -> None:
         ds = Dataset.from_dict(
             {
                 "coords": {
@@ -388,15 +532,23 @@ class TestDecodeCF:
             }
         )
 
-        dsc = conventions.decode_cf(ds)
-        assert dsc.timedelta.dtype == np.dtype("m8[ns]")
-        assert dsc.time.dtype == np.dtype("M8[ns]")
+        dsc = conventions.decode_cf(
+            ds,
+            decode_times=CFDatetimeCoder(time_unit=time_unit),
+            decode_timedelta=CFTimedeltaCoder(time_unit=time_unit),
+        )
+        assert dsc.timedelta.dtype == np.dtype(f"m8[{time_unit}]")
+        assert dsc.time.dtype == np.dtype(f"M8[{time_unit}]")
         dsc = conventions.decode_cf(ds, decode_times=False)
         assert dsc.timedelta.dtype == np.dtype("int64")
         assert dsc.time.dtype == np.dtype("int64")
-        dsc = conventions.decode_cf(ds, decode_times=True, decode_timedelta=False)
+        dsc = conventions.decode_cf(
+            ds,
+            decode_times=CFDatetimeCoder(time_unit=time_unit),
+            decode_timedelta=False,
+        )
         assert dsc.timedelta.dtype == np.dtype("int64")
-        assert dsc.time.dtype == np.dtype("M8[ns]")
+        assert dsc.time.dtype == np.dtype(f"M8[{time_unit}]")
         dsc = conventions.decode_cf(ds, decode_times=False, decode_timedelta=True)
         assert dsc.timedelta.dtype == np.dtype("m8[ns]")
         assert dsc.time.dtype == np.dtype("int64")
@@ -469,7 +621,7 @@ def test_decode_cf_variable_datetime64():
 
 @requires_cftime
 def test_decode_cf_variable_cftime():
-    variable = Variable(["time"], cftime_range("2000", periods=2))
+    variable = Variable(["time"], date_range("2000", periods=2, use_cftime=True))
     decoded = conventions.decode_cf_variable("time", variable)
     assert decoded.encoding == {}
     assert_identical(decoded, variable)
@@ -495,10 +647,22 @@ def test_encode_cf_variable_with_vlen_dtype() -> None:
     )
     encoded_v = conventions.encode_cf_variable(v)
     assert encoded_v.data.dtype.kind == "O"
-    assert coding.strings.check_vlen_dtype(encoded_v.data.dtype) == str
+    assert coding.strings.check_vlen_dtype(encoded_v.data.dtype) is str
 
     # empty array
     v = Variable(["x"], np.array([], dtype=coding.strings.create_vlen_dtype(str)))
     encoded_v = conventions.encode_cf_variable(v)
     assert encoded_v.data.dtype.kind == "O"
-    assert coding.strings.check_vlen_dtype(encoded_v.data.dtype) == str
+    assert coding.strings.check_vlen_dtype(encoded_v.data.dtype) is str
+
+
+def test_decode_cf_variables_decode_timedelta_warning() -> None:
+    v = Variable(["time"], [1, 2], attrs={"units": "seconds"})
+    variables = {"a": v}
+
+    with warnings.catch_warnings():
+        warnings.filterwarnings("error", "decode_timedelta", FutureWarning)
+        conventions.decode_cf_variables(variables, {}, decode_timedelta=True)
+
+    with pytest.warns(FutureWarning, match="decode_timedelta"):
+        conventions.decode_cf_variables(variables, {})
