@@ -6,12 +6,21 @@ from textwrap import dedent
 import numpy as np
 import pandas as pd
 import pytest
-from numpy.core import defchararray
 
 import xarray as xr
 from xarray.core import formatting
+from xarray.core.indexes import Index
+from xarray.tests import requires_cftime, requires_dask, requires_netCDF4
 
-from . import requires_dask, requires_netCDF4
+
+class CustomIndex(Index):
+    names: tuple[str, ...]
+
+    def __init__(self, names: tuple[str, ...]):
+        self.names = names
+
+    def __repr__(self):
+        return f"CustomIndex(coords={self.names})"
 
 
 class TestFormatting:
@@ -108,9 +117,9 @@ class TestFormatting:
                 np.arange(4) * np.timedelta64(500, "ms"),
                 "00:00:00 00:00:00.500000 00:00:01 00:00:01.500000",
             ),
-            (pd.to_timedelta(["NaT", "0s", "1s", "NaT"]), "NaT 00:00:00 00:00:01 NaT"),
+            (pd.to_timedelta(["NaT", "0s", "1s", "NaT"]), "NaT 00:00:00 00:00:01 NaT"),  # type: ignore[arg-type, unused-ignore]
             (
-                pd.to_timedelta(["1 day 1 hour", "1 day", "0 hours"]),
+                pd.to_timedelta(["1 day 1 hour", "1 day", "0 hours"]),  # type: ignore[arg-type, unused-ignore]
                 "1 days 01:00:00 1 days 00:00:00 0 days 00:00:00",
             ),
             ([1, 2, 3], "1 2 3"),
@@ -219,31 +228,50 @@ class TestFormatting:
         assert "\n" not in newlines
         assert "\t" not in tabs
 
-    def test_index_repr(self):
-        from xarray.core.indexes import Index
-
-        class CustomIndex(Index):
-            def __init__(self, names):
-                self.names = names
-
-            def __repr__(self):
-                return f"CustomIndex(coords={self.names})"
-
-        coord_names = ["x", "y"]
+    def test_index_repr(self) -> None:
+        coord_names = ("x", "y")
         index = CustomIndex(coord_names)
-        name = "x"
+        names = ("x",)
 
-        normal = formatting.summarize_index(name, index, col_width=20)
-        assert name in normal
+        normal = formatting.summarize_index(names, index, col_width=20)
+        assert names[0] in normal
+        assert len(normal.splitlines()) == len(names)
         assert "CustomIndex" in normal
 
-        CustomIndex._repr_inline_ = (
-            lambda self, max_width: f"CustomIndex[{', '.join(self.names)}]"
-        )
-        inline = formatting.summarize_index(name, index, col_width=20)
-        assert name in inline
+        class IndexWithInlineRepr(CustomIndex):
+            def _repr_inline_(self, max_width: int):
+                return f"CustomIndex[{', '.join(self.names)}]"
+
+        index = IndexWithInlineRepr(coord_names)
+        inline = formatting.summarize_index(names, index, col_width=20)
+        assert names[0] in inline
         assert index._repr_inline_(max_width=40) in inline
 
+    @pytest.mark.parametrize(
+        "names",
+        (
+            ("x",),
+            ("x", "y"),
+            ("x", "y", "z"),
+            ("x", "y", "z", "a"),
+        ),
+    )
+    def test_index_repr_grouping(self, names) -> None:
+        index = CustomIndex(names)
+
+        normal = formatting.summarize_index(names, index, col_width=20)
+        assert all(name in normal for name in names)
+        assert len(normal.splitlines()) == len(names)
+        assert "CustomIndex" in normal
+
+        hint_chars = [line[2] for line in normal.splitlines()]
+
+        if len(names) <= 1:
+            assert hint_chars == [" "]
+        else:
+            assert hint_chars[0] == "┌" and hint_chars[-1] == "└"
+            assert len(names) == 2 or hint_chars[1:-1] == ["│"] * (len(names) - 2)
+
     def test_diff_array_repr(self) -> None:
         da_a = xr.DataArray(
             np.array([[1, 2, 3], [4, 5, 6]], dtype="int64"),
@@ -267,7 +295,7 @@ class TestFormatting:
 
         byteorder = "<" if sys.byteorder == "little" else ">"
         expected = dedent(
-            """\
+            f"""\
         Left and right DataArray objects are not identical
         Differing dimensions:
             (x: 2, y: 3) != (x: 2)
@@ -278,18 +306,17 @@ class TestFormatting:
         R
             array([1, 2], dtype=int64)
         Differing coordinates:
-        L * x        (x) %cU1 'a' 'b'
-        R * x        (x) %cU1 'a' 'c'
+        L * x        (x) {byteorder}U1 8B 'a' 'b'
+        R * x        (x) {byteorder}U1 8B 'a' 'c'
         Coordinates only on the left object:
-          * y        (y) int64 1 2 3
+          * y        (y) int64 24B 1 2 3
         Coordinates only on the right object:
-            label    (x) int64 1 2
+            label    (x) int64 16B 1 2
         Differing attributes:
         L   units: m
         R   units: kg
         Attributes only on the left object:
             description: desc"""
-            % (byteorder, byteorder)
         )
 
         actual = formatting.diff_array_repr(da_a, da_b, "identical")
@@ -299,6 +326,51 @@ class TestFormatting:
             # depending on platform, dtype may not be shown in numpy array repr
             assert actual == expected.replace(", dtype=int64", "")
 
+        da_a = xr.DataArray(
+            np.array([[1, 2, 3], [4, 5, 6]], dtype="int8"),
+            dims=("x", "y"),
+            coords=xr.Coordinates(
+                {
+                    "x": np.array([True, False], dtype="bool"),
+                    "y": np.array([1, 2, 3], dtype="int16"),
+                },
+                indexes={"y": CustomIndex(("y",))},
+            ),
+        )
+
+        da_b = xr.DataArray(
+            np.array([1, 2], dtype="int8"),
+            dims="x",
+            coords=xr.Coordinates(
+                {
+                    "x": np.array([True, False], dtype="bool"),
+                    "label": ("x", np.array([1, 2], dtype="int16")),
+                },
+                indexes={"label": CustomIndex(("label",))},
+            ),
+        )
+
+        expected = dedent(
+            """\
+            Left and right DataArray objects are not equal
+            Differing dimensions:
+                (x: 2, y: 3) != (x: 2)
+            Differing values:
+            L
+                array([[1, 2, 3],
+                       [4, 5, 6]], dtype=int8)
+            R
+                array([1, 2], dtype=int8)
+            Coordinates only on the left object:
+              * y        (y) int16 6B 1 2 3
+            Coordinates only on the right object:
+              * label    (x) int16 4B 1 2
+            """.rstrip()
+        )
+
+        actual = formatting.diff_array_repr(da_a, da_b, "equals")
+        assert actual == expected
+
         va = xr.Variable(
             "x", np.array([1, 2, 3], dtype="int64"), {"title": "test Variable"}
         )
@@ -361,6 +433,36 @@ class TestFormatting:
         actual = formatting.diff_attrs_repr(attrs_a, attrs_c, "equals")
         assert expected == actual
 
+    def test__diff_mapping_repr_array_attrs_on_variables(self) -> None:
+        a = {
+            "a": xr.DataArray(
+                dims="x",
+                data=np.array([1], dtype="int16"),
+                attrs={"b": np.array([1, 2], dtype="int8")},
+            )
+        }
+        b = {
+            "a": xr.DataArray(
+                dims="x",
+                data=np.array([1], dtype="int16"),
+                attrs={"b": np.array([2, 3], dtype="int8")},
+            )
+        }
+        actual = formatting.diff_data_vars_repr(a, b, compat="identical", col_width=8)
+        expected = dedent(
+            """\
+            Differing data variables:
+            L   a   (x) int16 2B 1
+                Differing variable attributes:
+                    b: [1 2]
+            R   a   (x) int16 2B 1
+                Differing variable attributes:
+                    b: [2 3]
+            """.rstrip()
+        )
+
+        assert actual == expected
+
     def test_diff_dataset_repr(self) -> None:
         ds_a = xr.Dataset(
             data_vars={
@@ -368,62 +470,79 @@ class TestFormatting:
                 "var2": ("x", np.array([3, 4], dtype="int64")),
             },
             coords={
-                "x": np.array(["a", "b"], dtype="U1"),
+                "x": (
+                    "x",
+                    np.array(["a", "b"], dtype="U1"),
+                    {"foo": "bar", "same": "same"},
+                ),
                 "y": np.array([1, 2, 3], dtype="int64"),
             },
-            attrs={"units": "m", "description": "desc"},
+            attrs={"title": "mytitle", "description": "desc"},
         )
 
         ds_b = xr.Dataset(
             data_vars={"var1": ("x", np.array([1, 2], dtype="int64"))},
             coords={
-                "x": ("x", np.array(["a", "c"], dtype="U1"), {"source": 0}),
+                "x": (
+                    "x",
+                    np.array(["a", "c"], dtype="U1"),
+                    {"source": 0, "foo": "baz", "same": "same"},
+                ),
                 "label": ("x", np.array([1, 2], dtype="int64")),
             },
-            attrs={"units": "kg"},
+            attrs={"title": "newtitle"},
         )
 
         byteorder = "<" if sys.byteorder == "little" else ">"
         expected = dedent(
-            """\
+            f"""\
         Left and right Dataset objects are not identical
         Differing dimensions:
             (x: 2, y: 3) != (x: 2)
         Differing coordinates:
-        L * x        (x) %cU1 'a' 'b'
-        R * x        (x) %cU1 'a' 'c'
-            source: 0
+        L * x        (x) {byteorder}U1 8B 'a' 'b'
+            Differing variable attributes:
+                foo: bar
+        R * x        (x) {byteorder}U1 8B 'a' 'c'
+            Differing variable attributes:
+                source: 0
+                foo: baz
         Coordinates only on the left object:
-          * y        (y) int64 1 2 3
+          * y        (y) int64 24B 1 2 3
         Coordinates only on the right object:
-            label    (x) int64 1 2
+            label    (x) int64 16B 1 2
         Differing data variables:
-        L   var1     (x, y) int64 1 2 3 4 5 6
-        R   var1     (x) int64 1 2
+        L   var1     (x, y) int64 48B 1 2 3 4 5 6
+        R   var1     (x) int64 16B 1 2
         Data variables only on the left object:
-            var2     (x) int64 3 4
+            var2     (x) int64 16B 3 4
         Differing attributes:
-        L   units: m
-        R   units: kg
+        L   title: mytitle
+        R   title: newtitle
         Attributes only on the left object:
             description: desc"""
-            % (byteorder, byteorder)
         )
 
         actual = formatting.diff_dataset_repr(ds_a, ds_b, "identical")
         assert actual == expected
 
     def test_array_repr(self) -> None:
-        ds = xr.Dataset(coords={"foo": [1, 2, 3], "bar": [1, 2, 3]})
-        ds[(1, 2)] = xr.DataArray([0], dims="test")
+        ds = xr.Dataset(
+            coords={
+                "foo": np.array([1, 2, 3], dtype=np.uint64),
+                "bar": np.array([1, 2, 3], dtype=np.uint64),
+            }
+        )
+        ds[(1, 2)] = xr.DataArray(np.array([0], dtype=np.uint64), dims="test")
         ds_12 = ds[(1, 2)]
 
         # Test repr function behaves correctly:
         actual = formatting.array_repr(ds_12)
+
         expected = dedent(
             """\
-        <xarray.DataArray (1, 2) (test: 1)>
-        array([0])
+        <xarray.DataArray (1, 2) (test: 1)> Size: 8B
+        array([0], dtype=uint64)
         Dimensions without coordinates: test"""
         )
 
@@ -441,7 +560,7 @@ class TestFormatting:
             actual = formatting.array_repr(ds[(1, 2)])
             expected = dedent(
                 """\
-            <xarray.DataArray (1, 2) (test: 1)>
+            <xarray.DataArray (1, 2) (test: 1)> Size: 8B
             0
             Dimensions without coordinates: test"""
             )
@@ -459,7 +578,7 @@ class TestFormatting:
     def test_array_repr_recursive(self) -> None:
         # GH:issue:7111
 
-        # direct recurion
+        # direct recursion
         var = xr.Variable("x", [0, 1])
         var.attrs["x"] = var
         formatting.array_repr(var)
@@ -493,10 +612,112 @@ class TestFormatting:
 
         # Test numpy arrays raises:
         var = xr.DataArray([0.1, 0.2])
-        with pytest.raises(NotImplementedError) as excinfo:  # type: ignore
+        with pytest.raises(NotImplementedError) as excinfo:  # type: ignore[assignment]
             format(var, ".2f")
         assert "Using format_spec is only supported" in str(excinfo.value)
 
+    def test_datatree_print_empty_node(self):
+        dt: xr.DataTree = xr.DataTree(name="root")
+        printout = str(dt)
+        assert printout == "<xarray.DataTree 'root'>\nGroup: /"
+
+    def test_datatree_print_empty_node_with_attrs(self):
+        dat = xr.Dataset(attrs={"note": "has attrs"})
+        dt: xr.DataTree = xr.DataTree(name="root", dataset=dat)
+        printout = str(dt)
+        assert printout == dedent(
+            """\
+            <xarray.DataTree 'root'>
+            Group: /
+                Attributes:
+                    note:     has attrs"""
+        )
+
+    def test_datatree_print_node_with_data(self):
+        dat = xr.Dataset({"a": [0, 2]})
+        dt: xr.DataTree = xr.DataTree(name="root", dataset=dat)
+        printout = str(dt)
+        expected = [
+            "<xarray.DataTree 'root'>",
+            "Group: /",
+            "Dimensions",
+            "Coordinates",
+            "a",
+        ]
+        for expected_line, printed_line in zip(
+            expected, printout.splitlines(), strict=True
+        ):
+            assert expected_line in printed_line
+
+    def test_datatree_printout_nested_node(self):
+        dat = xr.Dataset({"a": [0, 2]})
+        root = xr.DataTree.from_dict(
+            {
+                "/results": dat,
+            }
+        )
+        printout = str(root)
+        assert printout.splitlines()[3].startswith("    ")
+
+    def test_datatree_repr_of_node_with_data(self):
+        dat = xr.Dataset({"a": [0, 2]})
+        dt: xr.DataTree = xr.DataTree(name="root", dataset=dat)
+        assert "Coordinates" in repr(dt)
+
+    def test_diff_datatree_repr_different_groups(self):
+        dt_1: xr.DataTree = xr.DataTree.from_dict({"a": None})
+        dt_2: xr.DataTree = xr.DataTree.from_dict({"b": None})
+
+        expected = dedent(
+            """\
+            Left and right DataTree objects are not identical
+
+            Children at root node do not match: ['a'] vs ['b']"""
+        )
+        actual = formatting.diff_datatree_repr(dt_1, dt_2, "identical")
+        assert actual == expected
+
+    def test_diff_datatree_repr_different_subgroups(self):
+        dt_1: xr.DataTree = xr.DataTree.from_dict({"a": None, "a/b": None, "a/c": None})
+        dt_2: xr.DataTree = xr.DataTree.from_dict({"a": None, "a/b": None})
+
+        expected = dedent(
+            """\
+            Left and right DataTree objects are not isomorphic
+
+            Children at node 'a' do not match: ['b', 'c'] vs ['b']"""
+        )
+        actual = formatting.diff_datatree_repr(dt_1, dt_2, "isomorphic")
+        assert actual == expected
+
+    def test_diff_datatree_repr_node_data(self):
+        # casting to int64 explicitly ensures that int64s are created on all architectures
+        ds1 = xr.Dataset({"u": np.int64(0), "v": np.int64(1)})
+        ds3 = xr.Dataset({"w": np.int64(5)})
+        dt_1: xr.DataTree = xr.DataTree.from_dict({"a": ds1, "a/b": ds3})
+        ds2 = xr.Dataset({"u": np.int64(0)})
+        ds4 = xr.Dataset({"w": np.int64(6)})
+        dt_2: xr.DataTree = xr.DataTree.from_dict({"a": ds2, "a/b": ds4}, name="foo")
+
+        expected = dedent(
+            """\
+            Left and right DataTree objects are not identical
+
+            Differing names:
+                None != 'foo'
+
+            Data at node 'a' does not match:
+                Data variables only on the left object:
+                    v        int64 8B 1
+
+            Data at node 'a/b' does not match:
+                Differing data variables:
+                L   w        int64 8B 5
+                R   w        int64 8B 6"""
+        )
+        actual = formatting.diff_datatree_repr(dt_1, dt_2, "identical")
+        assert actual == expected
+
 
 def test_inline_variable_array_repr_custom_repr() -> None:
     class CustomArray:
@@ -511,7 +732,7 @@ def test_inline_variable_array_repr_custom_repr() -> None:
 
             return formatted
 
-        def __array_function__(self, *args, **kwargs):
+        def __array_namespace__(self, *args, **kwargs):
             return NotImplemented
 
         @property
@@ -543,7 +764,7 @@ def test_set_numpy_options() -> None:
     assert np.get_printoptions() == original_options
 
 
-def test_short_numpy_repr() -> None:
+def test_short_array_repr() -> None:
     cases = [
         np.random.randn(500),
         np.random.randn(20, 20),
@@ -553,20 +774,19 @@ def test_short_numpy_repr() -> None:
     ]
     # number of lines:
     # for default numpy repr: 167, 140, 254, 248, 599
-    # for short_numpy_repr: 1, 7, 24, 19, 25
+    # for short_array_repr: 1, 7, 24, 19, 25
     for array in cases:
-        num_lines = formatting.short_numpy_repr(array).count("\n") + 1
+        num_lines = formatting.short_array_repr(array).count("\n") + 1
         assert num_lines < 30
 
     # threshold option (default: 200)
     array2 = np.arange(100)
-    assert "..." not in formatting.short_numpy_repr(array2)
+    assert "..." not in formatting.short_array_repr(array2)
     with xr.set_options(display_values_threshold=10):
-        assert "..." in formatting.short_numpy_repr(array2)
+        assert "..." in formatting.short_array_repr(array2)
 
 
 def test_large_array_repr_length() -> None:
-
     da = xr.DataArray(np.random.randn(100, 5, 1))
 
     result = repr(da).splitlines()
@@ -578,13 +798,14 @@ def test_repr_file_collapsed(tmp_path) -> None:
     arr_to_store = xr.DataArray(np.arange(300, dtype=np.int64), dims="test")
     arr_to_store.to_netcdf(tmp_path / "test.nc", engine="netcdf4")
 
-    with xr.open_dataarray(tmp_path / "test.nc") as arr, xr.set_options(
-        display_expand_data=False
+    with (
+        xr.open_dataarray(tmp_path / "test.nc") as arr,
+        xr.set_options(display_expand_data=False),
     ):
         actual = repr(arr)
         expected = dedent(
             """\
-        <xarray.DataArray (test: 300)>
+        <xarray.DataArray (test: 300)> Size: 2kB
         [300 values with dtype=int64]
         Dimensions without coordinates: test"""
         )
@@ -595,7 +816,7 @@ def test_repr_file_collapsed(tmp_path) -> None:
         actual = arr_loaded.__repr__()
         expected = dedent(
             """\
-        <xarray.DataArray (test: 300)>
+        <xarray.DataArray (test: 300)> Size: 2kB
         0 1 2 3 4 5 6 7 8 9 10 11 12 ... 288 289 290 291 292 293 294 295 296 297 298 299
         Dimensions without coordinates: test"""
         )
@@ -609,24 +830,24 @@ def test_repr_file_collapsed(tmp_path) -> None:
 )
 def test__mapping_repr(display_max_rows, n_vars, n_attr) -> None:
     long_name = "long_name"
-    a = defchararray.add(long_name, np.arange(0, n_vars).astype(str))
-    b = defchararray.add("attr_", np.arange(0, n_attr).astype(str))
-    c = defchararray.add("coord", np.arange(0, n_vars).astype(str))
-    attrs = {k: 2 for k in b}
-    coords = {_c: np.array([0, 1]) for _c in c}
+    a = np.char.add(long_name, np.arange(0, n_vars).astype(str))
+    b = np.char.add("attr_", np.arange(0, n_attr).astype(str))
+    c = np.char.add("coord", np.arange(0, n_vars).astype(str))
+    attrs = dict.fromkeys(b, 2)
+    coords = {_c: np.array([0, 1], dtype=np.uint64) for _c in c}
     data_vars = dict()
-    for (v, _c) in zip(a, coords.items()):
+    for v, _c in zip(a, coords.items(), strict=True):
         data_vars[v] = xr.DataArray(
             name=v,
-            data=np.array([3, 4]),
+            data=np.array([3, 4], dtype=np.uint64),
             dims=[_c[0]],
             coords=dict([_c]),
         )
+
     ds = xr.Dataset(data_vars)
     ds.attrs = attrs
 
     with xr.set_options(display_max_rows=display_max_rows):
-
         # Parse the data_vars print and show only data_vars rows:
         summary = formatting.dataset_repr(ds).split("\n")
         summary = [v for v in summary if long_name in v]
@@ -657,10 +878,11 @@ def test__mapping_repr(display_max_rows, n_vars, n_attr) -> None:
         col_width = formatting._calculate_col_width(ds.variables)
         dims_start = formatting.pretty_print("Dimensions:", col_width)
         dims_values = formatting.dim_summary_limited(
-            ds, col_width=col_width + 1, max_rows=display_max_rows
+            ds.sizes, col_width=col_width + 1, max_rows=display_max_rows
         )
+        expected_size = "1kB"
         expected = f"""\
-<xarray.Dataset>
+<xarray.Dataset> Size: {expected_size}
 {dims_start}({dims_values})
 Coordinates: ({n_vars})
 Data variables: ({n_vars})
@@ -716,7 +938,9 @@ def test_lazy_array_wont_compute() -> None:
     from xarray.core.indexing import LazilyIndexedArray
 
     class LazilyIndexedArrayNotComputable(LazilyIndexedArray):
-        def __array__(self, dtype=None):
+        def __array__(
+            self, dtype: np.typing.DTypeLike = None, /, *, copy: bool | None = None
+        ) -> np.ndarray:
             raise NotImplementedError("Computing this array is not possible.")
 
     arr = LazilyIndexedArrayNotComputable(np.array([1, 2]))
@@ -725,3 +949,219 @@ def test_lazy_array_wont_compute() -> None:
     # These will crash if var.data are converted to numpy arrays:
     var.__repr__()
     var._repr_html_()
+
+
+@pytest.mark.parametrize("as_dataset", (False, True))
+def test_format_xindexes_none(as_dataset: bool) -> None:
+    # ensure repr for empty xindexes can be displayed #8367
+
+    expected = """\
+    Indexes:
+        *empty*"""
+    expected = dedent(expected)
+
+    obj: xr.DataArray | xr.Dataset = xr.DataArray()
+    obj = obj._to_temp_dataset() if as_dataset else obj
+
+    actual = repr(obj.xindexes)
+    assert actual == expected
+
+
+@pytest.mark.parametrize("as_dataset", (False, True))
+def test_format_xindexes(as_dataset: bool) -> None:
+    expected = """\
+    Indexes:
+        x        PandasIndex"""
+    expected = dedent(expected)
+
+    obj: xr.DataArray | xr.Dataset = xr.DataArray([1], coords={"x": [1]})
+    obj = obj._to_temp_dataset() if as_dataset else obj
+
+    actual = repr(obj.xindexes)
+    assert actual == expected
+
+
+@requires_cftime
+def test_empty_cftimeindex_repr() -> None:
+    index = xr.coding.cftimeindex.CFTimeIndex([])
+
+    expected = """\
+    Indexes:
+        time     CFTimeIndex([], dtype='object', length=0, calendar=None, freq=None)"""
+    expected = dedent(expected)
+
+    da = xr.DataArray([], coords={"time": index})
+
+    actual = repr(da.indexes)
+    assert actual == expected
+
+
+def test_display_nbytes() -> None:
+    xds = xr.Dataset(
+        {
+            "foo": np.arange(1200, dtype=np.int16),
+            "bar": np.arange(111, dtype=np.int16),
+        }
+    )
+
+    # Note: int16 is used to ensure that dtype is shown in the
+    # numpy array representation for all OSes included Windows
+
+    actual = repr(xds)
+    expected = """
+<xarray.Dataset> Size: 3kB
+Dimensions:  (foo: 1200, bar: 111)
+Coordinates:
+  * foo      (foo) int16 2kB 0 1 2 3 4 5 6 ... 1194 1195 1196 1197 1198 1199
+  * bar      (bar) int16 222B 0 1 2 3 4 5 6 7 ... 104 105 106 107 108 109 110
+Data variables:
+    *empty*
+    """.strip()
+    assert actual == expected
+
+    actual = repr(xds["foo"])
+    array_repr = repr(xds.foo.data).replace("\n     ", "")
+    expected = f"""
+<xarray.DataArray 'foo' (foo: 1200)> Size: 2kB
+{array_repr}
+Coordinates:
+  * foo      (foo) int16 2kB 0 1 2 3 4 5 6 ... 1194 1195 1196 1197 1198 1199
+""".strip()
+    assert actual == expected
+
+
+def test_array_repr_dtypes():
+    # These dtypes are expected to be represented similarly
+    # on Ubuntu, macOS and Windows environments of the CI.
+    # Unsigned integer could be used as easy replacements
+    # for tests where the data-type does not matter,
+    # but the repr does, including the size
+    # (size of a int == size of an uint)
+
+    # Signed integer dtypes
+
+    ds = xr.DataArray(np.array([0], dtype="int8"), dims="x")
+    actual = repr(ds)
+    expected = """
+<xarray.DataArray (x: 1)> Size: 1B
+array([0], dtype=int8)
+Dimensions without coordinates: x
+        """.strip()
+    assert actual == expected
+
+    ds = xr.DataArray(np.array([0], dtype="int16"), dims="x")
+    actual = repr(ds)
+    expected = """
+<xarray.DataArray (x: 1)> Size: 2B
+array([0], dtype=int16)
+Dimensions without coordinates: x
+        """.strip()
+    assert actual == expected
+
+    # Unsigned integer dtypes
+
+    ds = xr.DataArray(np.array([0], dtype="uint8"), dims="x")
+    actual = repr(ds)
+    expected = """
+<xarray.DataArray (x: 1)> Size: 1B
+array([0], dtype=uint8)
+Dimensions without coordinates: x
+        """.strip()
+    assert actual == expected
+
+    ds = xr.DataArray(np.array([0], dtype="uint16"), dims="x")
+    actual = repr(ds)
+    expected = """
+<xarray.DataArray (x: 1)> Size: 2B
+array([0], dtype=uint16)
+Dimensions without coordinates: x
+        """.strip()
+    assert actual == expected
+
+    ds = xr.DataArray(np.array([0], dtype="uint32"), dims="x")
+    actual = repr(ds)
+    expected = """
+<xarray.DataArray (x: 1)> Size: 4B
+array([0], dtype=uint32)
+Dimensions without coordinates: x
+        """.strip()
+    assert actual == expected
+
+    ds = xr.DataArray(np.array([0], dtype="uint64"), dims="x")
+    actual = repr(ds)
+    expected = """
+<xarray.DataArray (x: 1)> Size: 8B
+array([0], dtype=uint64)
+Dimensions without coordinates: x
+        """.strip()
+    assert actual == expected
+
+    # Float dtypes
+
+    ds = xr.DataArray(np.array([0.0]), dims="x")
+    actual = repr(ds)
+    expected = """
+<xarray.DataArray (x: 1)> Size: 8B
+array([0.])
+Dimensions without coordinates: x
+        """.strip()
+    assert actual == expected
+
+    ds = xr.DataArray(np.array([0], dtype="float16"), dims="x")
+    actual = repr(ds)
+    expected = """
+<xarray.DataArray (x: 1)> Size: 2B
+array([0.], dtype=float16)
+Dimensions without coordinates: x
+        """.strip()
+    assert actual == expected
+
+    ds = xr.DataArray(np.array([0], dtype="float32"), dims="x")
+    actual = repr(ds)
+    expected = """
+<xarray.DataArray (x: 1)> Size: 4B
+array([0.], dtype=float32)
+Dimensions without coordinates: x
+        """.strip()
+    assert actual == expected
+
+    ds = xr.DataArray(np.array([0], dtype="float64"), dims="x")
+    actual = repr(ds)
+    expected = """
+<xarray.DataArray (x: 1)> Size: 8B
+array([0.])
+Dimensions without coordinates: x
+        """.strip()
+    assert actual == expected
+
+    # Signed integer dtypes
+
+    array = np.array([0])
+    ds = xr.DataArray(array, dims="x")
+    actual = repr(ds)
+    expected = f"""
+<xarray.DataArray (x: 1)> Size: {array.dtype.itemsize}B
+{array!r}
+Dimensions without coordinates: x
+        """.strip()
+    assert actual == expected
+
+    array = np.array([0], dtype="int32")
+    ds = xr.DataArray(array, dims="x")
+    actual = repr(ds)
+    expected = f"""
+<xarray.DataArray (x: 1)> Size: 4B
+{array!r}
+Dimensions without coordinates: x
+        """.strip()
+    assert actual == expected
+
+    array = np.array([0], dtype="int64")
+    ds = xr.DataArray(array, dims="x")
+    actual = repr(ds)
+    expected = f"""
+<xarray.DataArray (x: 1)> Size: 8B
+{array!r}
+Dimensions without coordinates: x
+        """.strip()
+    assert actual == expected
