@@ -1,4 +1,7 @@
+from __future__ import annotations
+
 import itertools
+from typing import Any
 
 import numpy as np
 import pandas as pd
@@ -6,19 +9,50 @@ import pytest
 
 from xarray import DataArray, Dataset, Variable
 from xarray.core import indexing, nputils
-
-from . import IndexerMaker, ReturnItem, assert_array_equal
+from xarray.core.indexes import PandasIndex, PandasMultiIndex
+from xarray.core.types import T_Xarray
+from xarray.tests import (
+    IndexerMaker,
+    ReturnItem,
+    assert_array_equal,
+    assert_identical,
+    raise_if_dask_computes,
+    requires_dask,
+)
+from xarray.tests.arrays import DuckArrayWrapper
 
 B = IndexerMaker(indexing.BasicIndexer)
 
 
+class TestIndexCallable:
+    def test_getitem(self):
+        def getter(key):
+            return key * 2
+
+        indexer = indexing.IndexCallable(getter)
+        assert indexer[3] == 6
+        assert indexer[0] == 0
+        assert indexer[-1] == -2
+
+    def test_setitem(self):
+        def getter(key):
+            return key * 2
+
+        def setter(key, value):
+            raise NotImplementedError("Setter not implemented")
+
+        indexer = indexing.IndexCallable(getter, setter)
+        with pytest.raises(NotImplementedError):
+            indexer[3] = 6
+
+
 class TestIndexers:
     def set_to_zero(self, x, i):
         x = x.copy()
         x[i] = 0
         return x
 
-    def test_expanded_indexer(self):
+    def test_expanded_indexer(self) -> None:
         x = np.random.randn(10, 11, 12, 13, 14)
         y = np.arange(5)
         arr = ReturnItem()
@@ -40,18 +74,7 @@ class TestIndexers:
         with pytest.raises(IndexError, match=r"too many indices"):
             indexing.expanded_indexer(arr[1, 2, 3], 2)
 
-    def test_asarray_tuplesafe(self):
-        res = indexing._asarray_tuplesafe(("a", 1))
-        assert isinstance(res, np.ndarray)
-        assert res.ndim == 0
-        assert res.item() == ("a", 1)
-
-        res = indexing._asarray_tuplesafe([(0,), (1,)])
-        assert res.shape == (2,)
-        assert res[0] == (0,)
-        assert res[1] == (1,)
-
-    def test_stacked_multiindex_min_max(self):
+    def test_stacked_multiindex_min_max(self) -> None:
         data = np.random.randn(3, 23, 4)
         da = DataArray(
             data,
@@ -66,64 +89,81 @@ class TestIndexers:
         assert_array_equal(da2.loc["a", s.max()], data[2, 22, 0])
         assert_array_equal(da2.loc["b", s.min()], data[0, 0, 1])
 
-    def test_convert_label_indexer(self):
-        # TODO: add tests that aren't just for edge cases
-        index = pd.Index([1, 2, 3])
-        with pytest.raises(KeyError, match=r"not all values found"):
-            indexing.convert_label_indexer(index, [0])
-        with pytest.raises(KeyError):
-            indexing.convert_label_indexer(index, 0)
-        with pytest.raises(ValueError, match=r"does not have a MultiIndex"):
-            indexing.convert_label_indexer(index, {"one": 0})
-
+    def test_group_indexers_by_index(self) -> None:
         mindex = pd.MultiIndex.from_product([["a", "b"], [1, 2]], names=("one", "two"))
-        with pytest.raises(KeyError, match=r"not all values found"):
-            indexing.convert_label_indexer(mindex, [0])
-        with pytest.raises(KeyError):
-            indexing.convert_label_indexer(mindex, 0)
-        with pytest.raises(ValueError):
-            indexing.convert_label_indexer(index, {"three": 0})
-        with pytest.raises(IndexError):
-            indexing.convert_label_indexer(mindex, (slice(None), 1, "no_level"))
+        data = DataArray(
+            np.zeros((4, 2, 2)), coords={"x": mindex, "y": [1, 2]}, dims=("x", "y", "z")
+        )
+        data.coords["y2"] = ("y", [2.0, 3.0])
 
-    def test_convert_label_indexer_datetime(self):
-        index = pd.to_datetime(["2000-01-01", "2001-01-01", "2002-01-01"])
-        actual = indexing.convert_label_indexer(index, "2001-01-01")
-        expected = (1, None)
-        assert actual == expected
+        grouped_indexers = indexing.group_indexers_by_index(
+            data, {"z": 0, "one": "a", "two": 1, "y": 0}, {}
+        )
 
-        actual = indexing.convert_label_indexer(index, index.to_numpy()[1])
-        assert actual == expected
+        for idx, indexers in grouped_indexers:
+            if idx is None:
+                assert indexers == {"z": 0}
+            elif idx.equals(data.xindexes["x"]):
+                assert indexers == {"one": "a", "two": 1}
+            elif idx.equals(data.xindexes["y"]):
+                assert indexers == {"y": 0}
+        assert len(grouped_indexers) == 3
 
-    def test_convert_unsorted_datetime_index_raises(self):
-        index = pd.to_datetime(["2001", "2000", "2002"])
-        with pytest.raises(KeyError):
-            # pandas will try to convert this into an array indexer. We should
-            # raise instead, so we can be sure the result of indexing with a
-            # slice is always a view.
-            indexing.convert_label_indexer(index, slice("2001", "2002"))
+        with pytest.raises(KeyError, match=r"no index found for coordinate 'y2'"):
+            indexing.group_indexers_by_index(data, {"y2": 2.0}, {})
+        with pytest.raises(
+            KeyError, match=r"'w' is not a valid dimension or coordinate"
+        ):
+            indexing.group_indexers_by_index(data, {"w": "a"}, {})
+        with pytest.raises(ValueError, match=r"cannot supply.*"):
+            indexing.group_indexers_by_index(data, {"z": 1}, {"method": "nearest"})
 
-    def test_get_dim_indexers(self):
-        mindex = pd.MultiIndex.from_product([["a", "b"], [1, 2]], names=("one", "two"))
-        mdata = DataArray(range(4), [("x", mindex)])
+    def test_map_index_queries(self) -> None:
+        def create_sel_results(
+            x_indexer,
+            x_index,
+            other_vars,
+            drop_coords,
+            drop_indexes,
+            rename_dims,
+        ):
+            dim_indexers = {"x": x_indexer}
+            index_vars = x_index.create_variables()
+            indexes = dict.fromkeys(index_vars, x_index)
+            variables = {}
+            variables.update(index_vars)
+            variables.update(other_vars)
 
-        dim_indexers = indexing.get_dim_indexers(mdata, {"one": "a", "two": 1})
-        assert dim_indexers == {"x": {"one": "a", "two": 1}}
+            return indexing.IndexSelResult(
+                dim_indexers=dim_indexers,
+                indexes=indexes,
+                variables=variables,
+                drop_coords=drop_coords,
+                drop_indexes=drop_indexes,
+                rename_dims=rename_dims,
+            )
 
-        with pytest.raises(ValueError, match=r"cannot combine"):
-            indexing.get_dim_indexers(mdata, {"x": "a", "two": 1})
+        def test_indexer(
+            data: T_Xarray,
+            x: Any,
+            expected: indexing.IndexSelResult,
+        ) -> None:
+            results = indexing.map_index_queries(data, {"x": x})
 
-        with pytest.raises(ValueError, match=r"do not exist"):
-            indexing.get_dim_indexers(mdata, {"y": "a"})
+            assert results.dim_indexers.keys() == expected.dim_indexers.keys()
+            assert_array_equal(results.dim_indexers["x"], expected.dim_indexers["x"])
 
-        with pytest.raises(ValueError, match=r"do not exist"):
-            indexing.get_dim_indexers(mdata, {"four": 1})
+            assert results.indexes.keys() == expected.indexes.keys()
+            for k in results.indexes:
+                assert results.indexes[k].equals(expected.indexes[k])
 
-    def test_remap_label_indexers(self):
-        def test_indexer(data, x, expected_pos, expected_idx=None):
-            pos, idx = indexing.remap_label_indexers(data, {"x": x})
-            assert_array_equal(pos.get("x"), expected_pos)
-            assert_array_equal(idx.get("x"), expected_idx)
+            assert results.variables.keys() == expected.variables.keys()
+            for k in results.variables:
+                assert_array_equal(results.variables[k], expected.variables[k])
+
+            assert set(results.drop_coords) == set(expected.drop_coords)
+            assert set(results.drop_indexes) == set(expected.drop_indexes)
+            assert results.rename_dims == expected.rename_dims
 
         data = Dataset({"x": ("x", [1, 2, 3])})
         mindex = pd.MultiIndex.from_product(
@@ -131,53 +171,98 @@ class TestIndexers:
         )
         mdata = DataArray(range(8), [("x", mindex)])
 
-        test_indexer(data, 1, 0)
-        test_indexer(data, np.int32(1), 0)
-        test_indexer(data, Variable([], 1), 0)
-        test_indexer(mdata, ("a", 1, -1), 0)
-        test_indexer(
-            mdata,
-            ("a", 1),
+        test_indexer(data, 1, indexing.IndexSelResult({"x": 0}))
+        test_indexer(data, np.int32(1), indexing.IndexSelResult({"x": 0}))
+        test_indexer(data, Variable([], 1), indexing.IndexSelResult({"x": 0}))
+        test_indexer(mdata, ("a", 1, -1), indexing.IndexSelResult({"x": 0}))
+
+        expected = create_sel_results(
             [True, True, False, False, False, False, False, False],
-            [-1, -2],
+            PandasIndex(pd.Index([-1, -2]), "three"),
+            {"one": Variable((), "a"), "two": Variable((), 1)},
+            ["x"],
+            ["one", "two"],
+            {"x": "three"},
         )
-        test_indexer(
-            mdata,
-            "a",
+        test_indexer(mdata, ("a", 1), expected)
+
+        expected = create_sel_results(
             slice(0, 4, None),
-            pd.MultiIndex.from_product([[1, 2], [-1, -2]]),
+            PandasMultiIndex(
+                pd.MultiIndex.from_product([[1, 2], [-1, -2]], names=("two", "three")),
+                "x",
+            ),
+            {"one": Variable((), "a")},
+            [],
+            ["one"],
+            {},
         )
-        test_indexer(
-            mdata,
-            ("a",),
+        test_indexer(mdata, "a", expected)
+
+        expected = create_sel_results(
             [True, True, True, True, False, False, False, False],
-            pd.MultiIndex.from_product([[1, 2], [-1, -2]]),
+            PandasMultiIndex(
+                pd.MultiIndex.from_product([[1, 2], [-1, -2]], names=("two", "three")),
+                "x",
+            ),
+            {"one": Variable((), "a")},
+            [],
+            ["one"],
+            {},
+        )
+        test_indexer(mdata, ("a",), expected)
+
+        test_indexer(
+            mdata, [("a", 1, -1), ("b", 2, -2)], indexing.IndexSelResult({"x": [0, 7]})
+        )
+        test_indexer(
+            mdata, slice("a", "b"), indexing.IndexSelResult({"x": slice(0, 8, None)})
         )
-        test_indexer(mdata, [("a", 1, -1), ("b", 2, -2)], [0, 7])
-        test_indexer(mdata, slice("a", "b"), slice(0, 8, None))
-        test_indexer(mdata, slice(("a", 1), ("b", 1)), slice(0, 6, None))
-        test_indexer(mdata, {"one": "a", "two": 1, "three": -1}, 0)
         test_indexer(
             mdata,
-            {"one": "a", "two": 1},
+            slice(("a", 1), ("b", 1)),
+            indexing.IndexSelResult({"x": slice(0, 6, None)}),
+        )
+        test_indexer(
+            mdata,
+            {"one": "a", "two": 1, "three": -1},
+            indexing.IndexSelResult({"x": 0}),
+        )
+
+        expected = create_sel_results(
             [True, True, False, False, False, False, False, False],
-            [-1, -2],
+            PandasIndex(pd.Index([-1, -2]), "three"),
+            {"one": Variable((), "a"), "two": Variable((), 1)},
+            ["x"],
+            ["one", "two"],
+            {"x": "three"},
         )
-        test_indexer(
-            mdata,
-            {"one": "a", "three": -1},
+        test_indexer(mdata, {"one": "a", "two": 1}, expected)
+
+        expected = create_sel_results(
             [True, False, True, False, False, False, False, False],
-            [1, 2],
+            PandasIndex(pd.Index([1, 2]), "two"),
+            {"one": Variable((), "a"), "three": Variable((), -1)},
+            ["x"],
+            ["one", "three"],
+            {"x": "two"},
         )
-        test_indexer(
-            mdata,
-            {"one": "a"},
+        test_indexer(mdata, {"one": "a", "three": -1}, expected)
+
+        expected = create_sel_results(
             [True, True, True, True, False, False, False, False],
-            pd.MultiIndex.from_product([[1, 2], [-1, -2]]),
+            PandasMultiIndex(
+                pd.MultiIndex.from_product([[1, 2], [-1, -2]], names=("two", "three")),
+                "x",
+            ),
+            {"one": Variable((), "a")},
+            [],
+            ["one"],
+            {},
         )
+        test_indexer(mdata, {"one": "a"}, expected)
 
-    def test_read_only_view(self):
-
+    def test_read_only_view(self) -> None:
         arr = DataArray(
             np.random.rand(3, 3),
             coords={"x": np.arange(3), "y": np.arange(3)},
@@ -190,7 +275,7 @@ class TestIndexers:
 
 
 class TestLazyArray:
-    def test_slice_slice(self):
+    def test_slice_slice(self) -> None:
         arr = ReturnItem()
         for size in [100, 99]:
             # We test even/odd size cases
@@ -220,7 +305,7 @@ class TestLazyArray:
                     actual = x[new_slice]
                     assert_array_equal(expected, actual)
 
-    def test_lazily_indexed_array(self):
+    def test_lazily_indexed_array(self) -> None:
         original = np.random.rand(10, 20, 30)
         x = indexing.NumpyIndexingAdapter(original)
         v = Variable(["i", "j", "k"], original)
@@ -245,10 +330,11 @@ class TestLazyArray:
                         assert expected.shape == actual.shape
                         assert_array_equal(expected, actual)
                         assert isinstance(actual._data, indexing.LazilyIndexedArray)
+                        assert isinstance(v_lazy._data, indexing.LazilyIndexedArray)
 
                         # make sure actual.key is appropriate type
                         if all(
-                            isinstance(k, (int, slice)) for k in v_lazy._data.key.tuple
+                            isinstance(k, int | slice) for k in v_lazy._data.key.tuple
                         ):
                             assert isinstance(v_lazy._data.key, indexing.BasicIndexer)
                         else:
@@ -265,29 +351,26 @@ class TestLazyArray:
             ([0, 3, 5], arr[:2]),
         ]
         for i, j in indexers:
-            expected = v[i][j]
+            expected_b = v[i][j]
             actual = v_lazy[i][j]
-            assert expected.shape == actual.shape
-            assert_array_equal(expected, actual)
+            assert expected_b.shape == actual.shape
+            assert_array_equal(expected_b, actual)
 
             # test transpose
             if actual.ndim > 1:
                 order = np.random.choice(actual.ndim, actual.ndim)
                 order = np.array(actual.dims)
                 transposed = actual.transpose(*order)
-                assert_array_equal(expected.transpose(*order), transposed)
+                assert_array_equal(expected_b.transpose(*order), transposed)
                 assert isinstance(
                     actual._data,
-                    (
-                        indexing.LazilyVectorizedIndexedArray,
-                        indexing.LazilyIndexedArray,
-                    ),
+                    indexing.LazilyVectorizedIndexedArray | indexing.LazilyIndexedArray,
                 )
 
             assert isinstance(actual._data, indexing.LazilyIndexedArray)
             assert isinstance(actual._data.array, indexing.NumpyIndexingAdapter)
 
-    def test_vectorized_lazily_indexed_array(self):
+    def test_vectorized_lazily_indexed_array(self) -> None:
         original = np.random.rand(10, 20, 30)
         x = indexing.NumpyIndexingAdapter(original)
         v_eager = Variable(["i", "j", "k"], x)
@@ -302,10 +385,7 @@ class TestLazyArray:
                 assert expected.shape == actual.shape
                 assert isinstance(
                     actual._data,
-                    (
-                        indexing.LazilyVectorizedIndexedArray,
-                        indexing.LazilyIndexedArray,
-                    ),
+                    indexing.LazilyVectorizedIndexedArray | indexing.LazilyIndexedArray,
                 )
                 assert_array_equal(expected, actual)
                 v_eager = expected
@@ -335,16 +415,50 @@ class TestLazyArray:
         ]
         check_indexing(v_eager, v_lazy, indexers)
 
+    def test_lazily_indexed_array_vindex_setitem(self) -> None:
+        lazy = indexing.LazilyIndexedArray(np.random.rand(10, 20, 30))
+
+        # vectorized indexing
+        indexer = indexing.VectorizedIndexer(
+            (np.array([0, 1]), np.array([0, 1]), slice(None, None, None))
+        )
+        with pytest.raises(
+            NotImplementedError,
+            match=r"Lazy item assignment with the vectorized indexer is not yet",
+        ):
+            lazy.vindex[indexer] = 0
+
+    @pytest.mark.parametrize(
+        "indexer_class, key, value",
+        [
+            (indexing.OuterIndexer, (0, 1, slice(None, None, None)), 10),
+            (indexing.BasicIndexer, (0, 1, slice(None, None, None)), 10),
+        ],
+    )
+    def test_lazily_indexed_array_setitem(self, indexer_class, key, value) -> None:
+        original = np.random.rand(10, 20, 30)
+        x = indexing.NumpyIndexingAdapter(original)
+        lazy = indexing.LazilyIndexedArray(x)
+
+        if indexer_class is indexing.BasicIndexer:
+            indexer = indexer_class(key)
+            lazy[indexer] = value
+        elif indexer_class is indexing.OuterIndexer:
+            indexer = indexer_class(key)
+            lazy.oindex[indexer] = value
+
+        assert_array_equal(original[key], value)
+
 
 class TestCopyOnWriteArray:
-    def test_setitem(self):
+    def test_setitem(self) -> None:
         original = np.arange(10)
         wrapped = indexing.CopyOnWriteArray(original)
         wrapped[B[:]] = 0
         assert_array_equal(original, np.arange(10))
         assert_array_equal(wrapped, np.zeros(10))
 
-    def test_sub_array(self):
+    def test_sub_array(self) -> None:
         original = np.arange(10)
         wrapped = indexing.CopyOnWriteArray(original)
         child = wrapped[B[:5]]
@@ -354,20 +468,20 @@ class TestCopyOnWriteArray:
         assert_array_equal(wrapped, np.arange(10))
         assert_array_equal(child, np.zeros(5))
 
-    def test_index_scalar(self):
+    def test_index_scalar(self) -> None:
         # regression test for GH1374
         x = indexing.CopyOnWriteArray(np.array(["foo", "bar"]))
         assert np.array(x[B[0]][B[()]]) == "foo"
 
 
 class TestMemoryCachedArray:
-    def test_wrapper(self):
+    def test_wrapper(self) -> None:
         original = indexing.LazilyIndexedArray(np.arange(10))
         wrapped = indexing.MemoryCachedArray(original)
         assert_array_equal(wrapped, np.arange(10))
         assert isinstance(wrapped.array, indexing.NumpyIndexingAdapter)
 
-    def test_sub_array(self):
+    def test_sub_array(self) -> None:
         original = indexing.LazilyIndexedArray(np.arange(10))
         wrapped = indexing.MemoryCachedArray(original)
         child = wrapped[B[:5]]
@@ -376,19 +490,19 @@ class TestMemoryCachedArray:
         assert isinstance(child.array, indexing.NumpyIndexingAdapter)
         assert isinstance(wrapped.array, indexing.LazilyIndexedArray)
 
-    def test_setitem(self):
+    def test_setitem(self) -> None:
         original = np.arange(10)
         wrapped = indexing.MemoryCachedArray(original)
         wrapped[B[:]] = 0
         assert_array_equal(original, np.zeros(10))
 
-    def test_index_scalar(self):
+    def test_index_scalar(self) -> None:
         # regression test for GH1374
         x = indexing.MemoryCachedArray(np.array(["foo", "bar"]))
         assert np.array(x[B[0]][B[()]]) == "foo"
 
 
-def test_base_explicit_indexer():
+def test_base_explicit_indexer() -> None:
     with pytest.raises(TypeError):
         indexing.ExplicitIndexer(())
 
@@ -404,7 +518,7 @@ def test_base_explicit_indexer():
     "indexer_cls",
     [indexing.BasicIndexer, indexing.OuterIndexer, indexing.VectorizedIndexer],
 )
-def test_invalid_for_all(indexer_cls):
+def test_invalid_for_all(indexer_cls) -> None:
     with pytest.raises(TypeError):
         indexer_cls(None)
     with pytest.raises(TypeError):
@@ -446,7 +560,7 @@ def check_array2d(indexer_cls):
     np.testing.assert_array_equal(value, array)
 
 
-def test_basic_indexer():
+def test_basic_indexer() -> None:
     check_integer(indexing.BasicIndexer)
     check_slice(indexing.BasicIndexer)
     with pytest.raises(TypeError):
@@ -455,7 +569,7 @@ def test_basic_indexer():
         check_array2d(indexing.BasicIndexer)
 
 
-def test_outer_indexer():
+def test_outer_indexer() -> None:
     check_integer(indexing.OuterIndexer)
     check_slice(indexing.OuterIndexer)
     check_array1d(indexing.OuterIndexer)
@@ -463,7 +577,7 @@ def test_outer_indexer():
         check_array2d(indexing.OuterIndexer)
 
 
-def test_vectorized_indexer():
+def test_vectorized_indexer() -> None:
     with pytest.raises(TypeError):
         check_integer(indexing.VectorizedIndexer)
     check_slice(indexing.VectorizedIndexer)
@@ -487,13 +601,15 @@ class Test_vectorized_indexer:
             slice(None),
         ]
 
-    def test_arrayize_vectorized_indexer(self):
+    def test_arrayize_vectorized_indexer(self) -> None:
         for i, j, k in itertools.product(self.indexers, repeat=3):
             vindex = indexing.VectorizedIndexer((i, j, k))
             vindex_array = indexing._arrayize_vectorized_indexer(
                 vindex, self.data.shape
             )
-            np.testing.assert_array_equal(self.data[vindex], self.data[vindex_array])
+            np.testing.assert_array_equal(
+                self.data.vindex[vindex], self.data.vindex[vindex_array]
+            )
 
         actual = indexing._arrayize_vectorized_indexer(
             indexing.VectorizedIndexer((slice(None),)), shape=(5,)
@@ -567,7 +683,7 @@ def get_indexers(shape, mode):
 @pytest.mark.parametrize(
     "sl", [slice(1, -1, 1), slice(None, -1, 2), slice(-1, 1, -1), slice(-1, 1, -2)]
 )
-def test_decompose_slice(size, sl):
+def test_decompose_slice(size, sl) -> None:
     x = np.arange(size)
     slice1, slice2 = indexing._decompose_slice(sl, size)
     expected = x[sl]
@@ -599,25 +715,48 @@ def test_decompose_slice(size, sl):
         indexing.IndexingSupport.VECTORIZED,
     ],
 )
-def test_decompose_indexers(shape, indexer_mode, indexing_support):
+def test_decompose_indexers(shape, indexer_mode, indexing_support) -> None:
     data = np.random.randn(*shape)
     indexer = get_indexers(shape, indexer_mode)
 
     backend_ind, np_ind = indexing.decompose_indexer(indexer, shape, indexing_support)
+    indexing_adapter = indexing.NumpyIndexingAdapter(data)
+
+    # Dispatch to appropriate indexing method
+    if indexer_mode.startswith("vectorized"):
+        expected = indexing_adapter.vindex[indexer]
+
+    elif indexer_mode.startswith("outer"):
+        expected = indexing_adapter.oindex[indexer]
+
+    else:
+        expected = indexing_adapter[indexer]  # Basic indexing
+
+    if isinstance(backend_ind, indexing.VectorizedIndexer):
+        array = indexing_adapter.vindex[backend_ind]
+    elif isinstance(backend_ind, indexing.OuterIndexer):
+        array = indexing_adapter.oindex[backend_ind]
+    else:
+        array = indexing_adapter[backend_ind]
 
-    expected = indexing.NumpyIndexingAdapter(data)[indexer]
-    array = indexing.NumpyIndexingAdapter(data)[backend_ind]
     if len(np_ind.tuple) > 0:
-        array = indexing.NumpyIndexingAdapter(array)[np_ind]
+        array_indexing_adapter = indexing.NumpyIndexingAdapter(array)
+        if isinstance(np_ind, indexing.VectorizedIndexer):
+            array = array_indexing_adapter.vindex[np_ind]
+        elif isinstance(np_ind, indexing.OuterIndexer):
+            array = array_indexing_adapter.oindex[np_ind]
+        else:
+            array = array_indexing_adapter[np_ind]
     np.testing.assert_array_equal(expected, array)
 
     if not all(isinstance(k, indexing.integer_types) for k in np_ind.tuple):
         combined_ind = indexing._combine_indexers(backend_ind, shape, np_ind)
-        array = indexing.NumpyIndexingAdapter(data)[combined_ind]
+        assert isinstance(combined_ind, indexing.VectorizedIndexer)
+        array = indexing_adapter.vindex[combined_ind]
         np.testing.assert_array_equal(expected, array)
 
 
-def test_implicit_indexing_adapter():
+def test_implicit_indexing_adapter() -> None:
     array = np.arange(10, dtype=np.int64)
     implicit = indexing.ImplicitToExplicitIndexingAdapter(
         indexing.NumpyIndexingAdapter(array), indexing.BasicIndexer
@@ -626,7 +765,7 @@ def test_implicit_indexing_adapter():
     np.testing.assert_array_equal(array, implicit[:])
 
 
-def test_implicit_indexing_adapter_copy_on_write():
+def test_implicit_indexing_adapter_copy_on_write() -> None:
     array = np.arange(10, dtype=np.int64)
     implicit = indexing.ImplicitToExplicitIndexingAdapter(
         indexing.CopyOnWriteArray(array)
@@ -634,7 +773,7 @@ def test_implicit_indexing_adapter_copy_on_write():
     assert isinstance(implicit[:], indexing.ImplicitToExplicitIndexingAdapter)
 
 
-def test_outer_indexer_consistency_with_broadcast_indexes_vectorized():
+def test_outer_indexer_consistency_with_broadcast_indexes_vectorized() -> None:
     def nonzero(x):
         if isinstance(x, np.ndarray) and x.dtype.kind == "b":
             x = x.nonzero()[0]
@@ -654,7 +793,6 @@ def test_outer_indexer_consistency_with_broadcast_indexes_vectorized():
         np.arange(10) < 5,
     ]
     for i, j, k in itertools.product(indexers, repeat=3):
-
         if isinstance(j, np.ndarray) and j.dtype.kind == "b":  # match size
             j = np.arange(20) < 4
         if isinstance(k, np.ndarray) and k.dtype.kind == "b":
@@ -672,7 +810,7 @@ def test_outer_indexer_consistency_with_broadcast_indexes_vectorized():
         np.testing.assert_array_equal(actual_data, expected_data)
 
 
-def test_create_mask_outer_indexer():
+def test_create_mask_outer_indexer() -> None:
     indexer = indexing.OuterIndexer((np.array([0, -1, 2]),))
     expected = np.array([False, True, False])
     actual = indexing.create_mask(indexer, (5,))
@@ -684,7 +822,7 @@ def test_create_mask_outer_indexer():
     np.testing.assert_array_equal(expected, actual)
 
 
-def test_create_mask_vectorized_indexer():
+def test_create_mask_vectorized_indexer() -> None:
     indexer = indexing.VectorizedIndexer((np.array([0, -1, 2]), np.array([0, 1, -1])))
     expected = np.array([False, True, True])
     actual = indexing.create_mask(indexer, (5,))
@@ -698,7 +836,7 @@ def test_create_mask_vectorized_indexer():
     np.testing.assert_array_equal(expected, actual)
 
 
-def test_create_mask_basic_indexer():
+def test_create_mask_basic_indexer() -> None:
     indexer = indexing.BasicIndexer((-1,))
     actual = indexing.create_mask(indexer, (3,))
     np.testing.assert_array_equal(True, actual)
@@ -708,7 +846,7 @@ def test_create_mask_basic_indexer():
     np.testing.assert_array_equal(False, actual)
 
 
-def test_create_mask_dask():
+def test_create_mask_dask() -> None:
     da = pytest.importorskip("dask.array")
 
     indexer = indexing.OuterIndexer((1, slice(2), np.array([0, -1, 2])))
@@ -719,23 +857,23 @@ def test_create_mask_dask():
     assert actual.chunks == ((1, 1), (2, 1))
     np.testing.assert_array_equal(expected, actual)
 
-    indexer = indexing.VectorizedIndexer(
+    indexer_vec = indexing.VectorizedIndexer(
         (np.array([0, -1, 2]), slice(None), np.array([0, 1, -1]))
     )
     expected = np.array([[False, True, True]] * 2).T
     actual = indexing.create_mask(
-        indexer, (5, 2), da.empty((3, 2), chunks=((3,), (2,)))
+        indexer_vec, (5, 2), da.empty((3, 2), chunks=((3,), (2,)))
     )
     assert isinstance(actual, da.Array)
     np.testing.assert_array_equal(expected, actual)
 
     with pytest.raises(ValueError):
-        indexing.create_mask(indexer, (5, 2), da.empty((5,), chunks=(1,)))
+        indexing.create_mask(indexer_vec, (5, 2), da.empty((5,), chunks=(1,)))
 
 
-def test_create_mask_error():
+def test_create_mask_error() -> None:
     with pytest.raises(TypeError, match=r"unexpected key type"):
-        indexing.create_mask((1, 2), (3, 4))
+        indexing.create_mask((1, 2), (3, 4))  # type: ignore[arg-type]
 
 
 @pytest.mark.parametrize(
@@ -750,14 +888,82 @@ def test_create_mask_error():
         (np.array([0, -1, -1, -1, 1]), np.array([0, 0, 0, 0, 1])),
     ],
 )
-def test_posify_mask_subindexer(indices, expected):
+def test_posify_mask_subindexer(indices, expected) -> None:
     actual = indexing._posify_mask_subindexer(indices)
     np.testing.assert_array_equal(expected, actual)
 
 
-def test_indexing_1d_object_array():
+class ArrayWithNamespace:
+    def __array_namespace__(self, version=None):
+        pass
+
+
+class ArrayWithArrayFunction:
+    def __array_function__(self, func, types, args, kwargs):
+        pass
+
+
+class ArrayWithNamespaceAndArrayFunction:
+    def __array_namespace__(self, version=None):
+        pass
+
+    def __array_function__(self, func, types, args, kwargs):
+        pass
+
+
+def as_dask_array(arr, chunks):
+    try:
+        import dask.array as da
+    except ImportError:
+        return None
+
+    return da.from_array(arr, chunks=chunks)
+
+
+@pytest.mark.parametrize(
+    ["array", "expected_type"],
+    (
+        pytest.param(
+            indexing.CopyOnWriteArray(np.array([1, 2])),
+            indexing.CopyOnWriteArray,
+            id="ExplicitlyIndexed",
+        ),
+        pytest.param(
+            np.array([1, 2]), indexing.NumpyIndexingAdapter, id="numpy.ndarray"
+        ),
+        pytest.param(
+            pd.Index([1, 2]), indexing.PandasIndexingAdapter, id="pandas.Index"
+        ),
+        pytest.param(
+            as_dask_array(np.array([1, 2]), chunks=(1,)),
+            indexing.DaskIndexingAdapter,
+            id="dask.array",
+            marks=requires_dask,
+        ),
+        pytest.param(
+            ArrayWithNamespace(), indexing.ArrayApiIndexingAdapter, id="array_api"
+        ),
+        pytest.param(
+            ArrayWithArrayFunction(),
+            indexing.NdArrayLikeIndexingAdapter,
+            id="array_like",
+        ),
+        pytest.param(
+            ArrayWithNamespaceAndArrayFunction(),
+            indexing.ArrayApiIndexingAdapter,
+            id="array_api_with_fallback",
+        ),
+    ),
+)
+def test_as_indexable(array, expected_type):
+    actual = indexing.as_indexable(array)
+
+    assert isinstance(actual, expected_type)
+
+
+def test_indexing_1d_object_array() -> None:
     items = (np.arange(3), np.arange(6))
-    arr = DataArray(np.array(items))
+    arr = DataArray(np.array(items, dtype=object))
 
     actual = arr[0]
 
@@ -766,3 +972,96 @@ def test_indexing_1d_object_array():
     expected = DataArray(expected_data)
 
     assert [actual.data.item()] == [expected.data.item()]
+
+
+@requires_dask
+def test_indexing_dask_array() -> None:
+    import dask.array
+
+    da = DataArray(
+        np.ones(10 * 3 * 3).reshape((10, 3, 3)),
+        dims=("time", "x", "y"),
+    ).chunk(dict(time=-1, x=1, y=1))
+    with raise_if_dask_computes():
+        actual = da.isel(time=dask.array.from_array([9], chunks=(1,)))
+    expected = da.isel(time=[9])
+    assert_identical(actual, expected)
+
+
+@requires_dask
+def test_indexing_dask_array_scalar() -> None:
+    # GH4276
+    import dask.array
+
+    a = dask.array.from_array(np.linspace(0.0, 1.0))
+    da = DataArray(a, dims="x")
+    x_selector = da.argmax(dim=...)
+    assert not isinstance(x_selector, DataArray)
+    with raise_if_dask_computes():
+        actual = da.isel(x_selector)
+    expected = da.isel(x=-1)
+    assert_identical(actual, expected)
+
+
+@requires_dask
+def test_vectorized_indexing_dask_array() -> None:
+    # https://github.com/pydata/xarray/issues/2511#issuecomment-563330352
+    darr = DataArray(data=[0.2, 0.4, 0.6], coords={"z": range(3)}, dims=("z",))
+    indexer = DataArray(
+        data=np.random.randint(0, 3, 8).reshape(4, 2).astype(int),
+        coords={"y": range(4), "x": range(2)},
+        dims=("y", "x"),
+    )
+    expected = darr[indexer]
+
+    # fails because we can't index pd.Index lazily (yet).
+    # We could make this succeed by auto-chunking the values
+    # and constructing a lazy index variable, and not automatically
+    # create an index for it.
+    with pytest.raises(ValueError, match="Cannot index with"):
+        with raise_if_dask_computes():
+            darr.chunk()[indexer.chunk({"y": 2})]
+    with pytest.raises(ValueError, match="Cannot index with"):
+        with raise_if_dask_computes():
+            actual = darr[indexer.chunk({"y": 2})]
+
+    with raise_if_dask_computes():
+        actual = darr.drop_vars("z").chunk()[indexer.chunk({"y": 2})]
+    assert_identical(actual, expected.drop_vars("z"))
+
+    with raise_if_dask_computes():
+        actual_variable = darr.variable.chunk()[indexer.variable.chunk({"y": 2})]
+    assert_identical(actual_variable, expected.variable)
+
+
+@requires_dask
+def test_advanced_indexing_dask_array() -> None:
+    # GH4663
+    import dask.array as da
+
+    ds = Dataset(
+        dict(
+            a=("x", da.from_array(np.random.randint(0, 100, 100))),
+            b=(("x", "y"), da.random.random((100, 10))),
+        )
+    )
+    expected = ds.b.sel(x=ds.a.compute())
+    with raise_if_dask_computes():
+        actual = ds.b.sel(x=ds.a)
+    assert_identical(expected, actual)
+
+    with raise_if_dask_computes():
+        actual = ds.b.sel(x=ds.a.data)
+    assert_identical(expected, actual)
+
+
+def test_backend_indexing_non_numpy() -> None:
+    """This model indexing of a Zarr store that reads to GPU memory."""
+    array = DuckArrayWrapper(np.array([1, 2, 3]))
+    indexed = indexing.explicit_indexing_adapter(
+        indexing.BasicIndexer((slice(1),)),
+        shape=array.shape,
+        indexing_support=indexing.IndexingSupport.BASIC,
+        raw_indexing_method=array.__getitem__,
+    )
+    np.testing.assert_array_equal(indexed.array, np.array([1]))
